// Copyright 2015-2017 Parity Technologies (UK) Ltd.
// This file is part of Parity.

// Parity is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Parity is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Parity.  If not, see <http://www.gnu.org/licenses/>.

import React, { Component, PropTypes } from 'react';
import { Checkbox } from 'material-ui';

import Form, { Input, InputAddressSelect, AddressSelect } from '~/ui/Form';
import { nullableProptype } from '~/util/proptypes';

import TokenSelect from './tokenSelect';
import styles from '../transfer.css';

const CHECK_STYLE = {
  position: 'absolute',
  top: '38px',
  left: '1em'
};

export default class Details extends Component {
  static propTypes = {
    address: PropTypes.string,
    balance: PropTypes.object,
    all: PropTypes.bool,
    extras: PropTypes.bool,
    sender: PropTypes.string,
    senderError: PropTypes.string,
    sendersBalances: PropTypes.object,
    recipient: PropTypes.string,
    recipientError: PropTypes.string,
    tag: PropTypes.string,
    total: PropTypes.string,
    totalError: PropTypes.string,
    value: PropTypes.string,
    valueError: PropTypes.string,
    onChange: PropTypes.func.isRequired,
    wallet: PropTypes.object,
    senders: nullableProptype(PropTypes.object)
  };

  static defaultProps = {
    wallet: null,
    senders: null
  };

  render () {
    const { all, extras, tag, total, totalError, value, valueError } = this.props;
    const label = `amount to transfer (in ${tag})`;

    return (
      <Form>
        { this.renderTokenSelect() }
        { this.renderFromAddress() }
        { this.renderToAddress() }
        <div className={ styles.columns }>
          <div>
            <Input
              disabled={ all }
              label={ label }
              hint='the amount to transfer to the recipient'
              value={ value }
              error={ valueError }
              onChange={ this.onEditValue }
            />
          </div>
          <div>
            <Checkbox
              checked={ all }
              label='full account balance'
              onCheck={ this.onCheckAll }
              style={ CHECK_STYLE }
            />
          </div>
        </div>
        <div className={ styles.columns }>
          <div>
            <Input
              disabled
              label='total transaction amount'
              error={ totalError }
            >
              <div className={ styles.inputoverride }>
                { total }<small> ETH</small>
              </div>
            </Input>
          </div>

          <div>
            <Checkbox
              checked={ extras }
              label='advanced sending options'
              onCheck={ this.onCheckExtras }
              style={ CHECK_STYLE }
            />
          </div>
        </div>
      </Form>
    );
  }

  renderFromAddress () {
    const { sender, senderError, senders, sendersBalances } = this.props;

    if (!senders) {
      return null;
    }

    return (
      <div className={ styles.address }>
        <AddressSelect
          accounts={ senders }
          error={ senderError }
          label='sender address'
          hint='the sender address'
          value={ sender }
          onChange={ this.onEditSender }
          balances={ sendersBalances }
        />
      </div>
    );
  }

  renderToAddress () {
    const { recipient, recipientError } = this.props;

    return (
      <div className={ styles.address }>
        <InputAddressSelect
          label='recipient address'
          hint='the recipient address'
          error={ recipientError }
          value={ recipient }
          onChange={ this.onEditRecipient }
        />
      </div>
    );
  }

  renderTokenSelect () {
    const { balance, tag } = this.props;

    return (
      <TokenSelect
        balance={ balance }
        tag={ tag }
        onChange={ this.onChangeToken }
      />
    );
  }

  onChangeToken = (event, index, tag) => {
    this.props.onChange('tag', tag);
  }

  onEditSender = (event, sender) => {
    this.props.onChange('sender', sender);
  }

  onEditRecipient = (event, recipient) => {
    this.props.onChange('recipient', recipient);
  }

  onEditValue = (event) => {
    this.props.onChange('value', event.target.value);
  }

  onCheckAll = () => {
    this.props.onChange('all', !this.props.all);
  }

  onCheckExtras = () => {
    this.props.onChange('extras', !this.props.extras);
  }

  onContacts = () => {
    this.setState({
      showAddresses: true
    });
  }
}
